@@ -7,23 +7,94 @@ use utf8;
 
 BEGIN {
 	$MooX::Struct::AUTHORITY = 'cpan:TOBYINK';
-	$MooX::Struct::VERSION   = '0.004';
+	$MooX::Struct::VERSION   = '0.005';
 }
 
-use Moo         1.000000;
-use Object::ID  0         qw( object_id );
+use Moo          1.000000;
+use Object::ID   0         qw( object_id );
+use Scalar::Does 0         qw( does );
+
+use overload
+	q[""]      => 'TO_STRING',
+	q[bool]    => sub { 1 },
+	q[@{}]     => 'TO_ARRAY',
+	q[=]       => 'CLONE',
+	fallback   => 1;
+
+METHODS: {
+	no warnings;
+	sub OBJECT_ID   { goto \&object_id };
+	sub FIELDS      { qw() };
+	sub TYPE        { +undef };
+	sub TO_ARRAY    {  [ map {;       $_[0]->$_ } $_[0]->FIELDS ] };
+	sub TO_HASH     { +{ map {; $_ => $_[0]->$_ } $_[0]->FIELDS } };
+	sub TO_STRING   { join q[ ], @{ $_[0]->TO_ARRAY } };
+	sub CLONE       { my $s = shift; ref($s)->new(%{$s->TO_HASH}, @_) };
+};
+
+sub BUILDARGS
+{
+	my $class = shift;
+	if (
+		@_ == 1                 and
+		does($_[0], 'ARRAY')    and
+		not does($_[0], 'HASH')
+	)
+	{
+		my @fields = $class->FIELDS;
+		my @values = @{ $_[0] };
+		Carp::confess("too many values passed to constructor (expected @fields); stopped")
+			unless @fields >= @values;
+		no warnings;
+		return +{
+			map {
+				$fields[$_] => $values[$_];
+			} 0 .. $#values
+		}
+	}
+	return $class->SUPER::BUILDARGS(@_);
+}
+
+my $done = 0;
+sub _data_printer
+{
+	require Data::Printer::Filter;
+	require Term::ANSIColor;
+	my $self   = shift;
+	
+	my @values = map { scalar Data::Printer::p($_) } @$self;
+
+	if (grep /\n/, @values)
+	{
+		return sprintf(
+			"%s[\n\t%s,\n]",
+			Term::ANSIColor::colored($self->TYPE, 'bright_yellow'),
+			join(qq[,\n\t], map { s/\n/\n\t/gm; $_ } @values),
+		);
+	}
+	else
+	{
+		return sprintf(
+			'%s[ %s ]',
+			Term::ANSIColor::colored($self->TYPE, 'bright_yellow'),
+			join(q[, ], @values),
+		);
+	}
+}
 
 BEGIN {
 	package MooX::Struct::Processor;
 	
+	sub _uniq { my %seen; grep { not $seen{$_}++ } @_ };
+	
 	use Moo                  1.000000;
 	use Carp                 0         qw( confess      );
 	use Data::OptList        0         qw(              );
 	use Sub::Install         0         qw( install_sub  );
-	use Scalar::Does         0         qw( does blessed );
+	use Scalar::Does         0         qw( does blessed looks_like_number );
 	use namespace::clean               qw(              );
 	use B::Hooks::EndOfScope           qw( on_scope_end );
-
+	
 	has flags => (
 		is       => 'ro',
 		isa      => sub { die "flags must be HASH" unless does $_[0], 'HASH' },
@@ -55,7 +126,7 @@ BEGIN {
 		$ENV{PERL_MOOX_STRUCT_TRACE}
 		or shift->flags->{trace};
 	}
-
+	
 	has trace_handle => (
 		is       => 'lazy',
 	);
@@ -65,7 +136,7 @@ BEGIN {
 		require IO::Handle;
 		\*STDERR;
 	}
-
+	
 	my $counter = 0;
 	sub create_class
 	{
@@ -104,6 +175,8 @@ BEGIN {
 					join(q[ ] => @parents),
 				);
 			}
+			
+			return map { $_->can('FIELDS') ? $_->FIELDS : () } @parents;
 		}
 		else
 		{
@@ -172,6 +245,15 @@ BEGIN {
 					unless does($_[0], 'HASH');
 			};
 		}
+		elsif ($name =~ /^\+(.+)/)
+		{
+			$name = $1;
+			$spec{isa} ||= sub {
+				die "wrong type for '$name' (not number)"
+					unless looks_like_number($_[0]);
+			};
+			$spec{default} ||= sub { 0 } unless $spec{required};
+		}
 		elsif ($name =~ /^\$(.+)/)
 		{
 			$name = $1;
@@ -209,7 +291,7 @@ BEGIN {
 				$self->trace_handle->printf("$code\n");
 			}
 		}
-
+		
 		Moo
 			->_constructor_maker_for($klass)
 			->register_attribute_specs($name, $spec);
@@ -220,10 +302,11 @@ BEGIN {
 			
 		Moo
 			->_maybe_reset_handlemoose($klass);
-
-		return;
+		
+		return $name;
 	}
 	
+	# returns a list of "fields" resulting from the argument
 	sub process_argument
 	{
 		my $self = shift;
@@ -240,16 +323,20 @@ BEGIN {
 	sub make_sub
 	{
 		my ($self, $subname, $proto) = @_;
-		return sub ()
+		return sub (;$)
 		{
 			1 if $] < 5.014; # bizarre, but necessary!
 			if (ref $proto)  # inflate!
 			{
-				my $klass = $self->create_class;
-				$self->process_argument($klass, @$_)
-					for @{ Data::OptList::mkopt($proto) };
+				my $klass  = $self->create_class;
+				my @fields = _uniq map {
+					$self->process_argument($klass, @$_)
+				} @{ Data::OptList::mkopt($proto) };
+				$self->process_method($klass, FIELDS => sub { @fields });
+				$self->process_method($klass, TYPE   => sub { $subname });
 				$proto = $klass;
 			}
+			return $proto->new(@_) if @_;
 			return $proto;
 		}
 	}
@@ -307,6 +394,9 @@ MooX::Struct - make simple lightweight record-like structures that make sounds l
  ;
  
  my $origin = Point3D->new( x => 0, y => 0, z => 0 );
+ 
+ # or...
+ my $origin = Point3D[ 0, 0, 0 ];
 
 =head1 DESCRIPTION
 
@@ -323,8 +413,7 @@ A struct is just an "anonymous" Moo class. MooX::Struct creates this class
 for you, and installs a lexical alias for it in your namespace. Thus your
 module can create a "Point3D" struct, and some other module can too, and
 they won't interfere with each other. All struct classes inherit from
-MooX::Struct; and MooX::Struct provides a useful method: C<object_id> (see
-L<Object::ID>).
+MooX::Struct.
 
 Arguments for MooX::Struct are key-value pairs, where keys are the struct
 names, and values are arrayrefs.
@@ -347,6 +436,7 @@ MooX::Struct itself.
 
 Structs can inherit from other structs, or from normal classes. If inheriting
 from another struct, then you I<must> define both in the same C<use> statement.
+Inheriting from a non-struct class is discouraged.
 
  # Not like this.
  use MooX::Struct Point   => [ 'x', 'y' ];
@@ -390,7 +480,9 @@ sigils of C<< @ >> and C<< % >> specify that the attribute isa arrayref or
 hashref respectively. (Blessed arrayrefs and hashrefs are accepted; as are
 objects which overload C<< @{} >> and C<< %{} >>.) The prefix sigil C<< $ >>
 specifies that the attribute value must not be an unblessed arrayref or hashref.
-The postfix sigil C<< ! >> specifies that the attribute is required.
+The prefix sigil C<< + >> indicates the attribute is a number, and provides
+a default value of 0, unless the attribute is required. The postfix sigil
+C<< ! >> specifies that the attribute is required.
 
  use MooX::Struct
     Person  => [qw( $name! @children )];
@@ -418,6 +510,99 @@ read-write rather than read-only.
 
 Flags C<< -trace >> and C<< -deparse >> may be of use debugging.
 
+=head2 Instantiating Structs
+
+There are two supported methods of instatiating structs. You can use a
+traditional class-like constructor with named parameters:
+
+ my $point = Point->new( x => 1, y => 2 );
+
+Or you can use the abbreviated syntax with positional parameters:
+
+ my $point = Point[ 1, 2 ];
+
+If you know about Moo and peek around in the source code for this module,
+then I'm sure you can figure out additional ways to instantiate them, but
+the above are the only supported two.
+
+When inheritance has been used, it might not always be clear what order
+the positional parameters come in (though see the documentation for the
+C<FIELDS> below), so the traditional class-like style may be preferred.
+
+=head2 Methods
+
+Structs are objects and thus have methods. The following methods are defined:
+
+=over
+
+=item C<OBJECT_ID> 
+
+Returns a unique identifier for the object.
+
+=item C<FIELDS> 
+
+Returns a list of fields associated with the object. For the C<Point3D> struct
+in the SYNPOSIS, this would be <c>'x', 'y', 'z'</c>.
+
+The order the fields are returned in is equal to the order they must be supplied
+for the positional constructor.
+
+=item C<TYPE>
+
+Returns the type name of the struct, e.g. <c>'Point3D'</c>.
+
+=item C<TO_HASH>
+
+Returns a reference to an unblessed hash where the object's fields are the
+keys and the object's values are the hash values.
+
+=item C<TO_ARRAY>
+
+Returns a reference to an unblessed array where the object's values are the
+array items, in the same order as listed by C<FIELDS>.
+
+=item C<TO_STRING>
+
+Joins C<TO_ARRAY> with whitespace. This is not necessarily a brilliant
+stringification, but easy enough to overload:
+
+ use MooX::Struct
+    Point => [
+       qw( x y ),
+       TO_STRING => sub {
+          sprintf "(%d, %d)"), $_[0]->x, $_[0]->y;
+       },
+    ]
+ ;
+
+=item C<CLONE>
+
+Creates a shallow clone of the object. 
+
+=item C<BUILDARGS>
+
+Moo internal fu.
+
+=item C<_data_printer>
+
+Automatic pretty printing with L<Data::Printer>.
+
+ use Data::Printer;
+ use MooX::Struct Point => [qw/ +x +y /];
+ my $origin = Point[];
+ p $origin;
+
+=back
+
+With the exception of C<FIELDS> and C<TYPE>, any of these can be overridden
+using the standard way of specifying methods for structs.
+
+=head2 Overloading
+
+MooX::Struct overloads stringification and array dereferencing. Objects always
+evaluate to true in a boolean context. (Even if they stringify to the empty
+string.)
+
 =head1 BUGS
 
 Please report any bugs to
